<*+CHANGESYM*>
<*+M2EXTENSIONS*>
<*+O2EXTENSIONS*>
<* ALIGNMENT = "8" *>
MODULE SciterDLL;
IMPORT Windows, SYSTEM;

TYPE
  HELEMENT* = Windows.PVOID;
  THTMLAYOUT_NOTIFY* = PROCEDURE ["StdCall"] (uMsg: Windows.UINT; wParam: Windows.WPARAM; lParam: Windows.LPARAM; vParam: Windows.PVOID): Windows.LRESULT;

  ElementEventProc* = PROCEDURE ["StdCall"] (tag: Windows.PVOID; he: HELEMENT; evtg: Windows.UINT; prms: Windows.PVOID);

  PSCITER_VALUE* = POINTER ["StdCall"] TO SCITER_VALUE;
  SCITER_VALUE* = RECORD
    t: Windows.UINT;
    u: Windows.UINT;
    d: SYSTEM.CARD64;
  END;

  PSCITER_CALLBACK_NOTIFICATION* = POINTER ["StdCall"] TO SCITER_CALLBACK_NOTIFICATION;
  SCITER_CALLBACK_NOTIFICATION* = RECORD 
    code*: Windows.UINT; -- < [in] one of the codes above.*/
    hwnd*: Windows.HWND; -- < [in] HWND of the window this callback was attached to.*/
  END;

  PSCN_LOAD_DATA* = POINTER ["StdCall"] TO SCN_LOAD_DATA;
  SCN_LOAD_DATA* = RECORD  (SCITER_CALLBACK_NOTIFICATION)
    uri*: Windows.PCWSTR;       -- < [in] Zero terminated string, fully qualified uri, for example "http://server/folder/file.ext".
    outData*:     Windows.PCBYTE;   -- < [in,out] pointer to loaded data to return. if data exists in the cache then this field contain pointer to it*/
    outDataSize*: Windows.UINT; -- < [in,out] loaded data size to return.*/
    dataType*:    Windows.UINT;    -- < [in] SciterResourceType */                              
    requestId*:   Windows.PVOID;  -- < [in] request id that needs to be passed as is to the SciterDataReadyAsync call */
    principal*:   HELEMENT;
    initiator*:   HELEMENT;
  END;

  PSCN_DATA_LOADED* = POINTER ["StdCall"] TO SCN_DATA_LOADED;
  SCN_DATA_LOADED* = RECORD (SCITER_CALLBACK_NOTIFICATION)
    uri*:      Windows.PCWSTR; -- < [in] zero terminated string, fully qualified uri, for example "http://server/folder/file.ext".
    data*:     Windows.PCBYTE; -- < [in] pointer to loaded data.
    dataSize*: Windows.DWORD;  -- < [in] loaded data size (in bytes).
    dataType*: Windows.UINT;   -- < [in] SciterResourceType 
    status*:   Windows.UINT;   -- < [in] status = 0 (dataSize == 0) - unknown error. 
                               --        status = 100..505 - http response status, Note: 200 - OK! 
                               --        status > 12000 - wininet error code, see ERROR_INTERNET_*** in wininet.h                                   
  END;

  PSCN_ATTACH_BEHAVIOR* = POINTER ["StdCall"] TO SCN_ATTACH_BEHAVIOR;
  SCN_ATTACH_BEHAVIOR* = RECORD(SCITER_CALLBACK_NOTIFICATION)
    element*:      HELEMENT;         -- < [in] target DOM element handle
    behaviorName*: Windows.PCSTR;    -- < [in] zero terminated string, string appears as value of CSS behavior:"???" attribute.
    elementProc*:  ElementEventProc; -- < [out] pointer to ElementEventProc function.
    elementTag*:   Windows.PVOID;    -- < [out] tag value, passed as is into pointer ElementEventProc function.
  END;

  PSCN_CALLBACK_HOST* = POINTER ["StdCall"] TO SCN_CALLBACK_HOST;
  SCN_CALLBACK_HOST* = RECORD(SCITER_CALLBACK_NOTIFICATION)
    channel*: Windows.UINT; -- 0 - stdin, 1 - stdout, 2 - stderr
    p1*:      SCITER_VALUE; -- in, parameter #1
    p2*:      SCITER_VALUE; -- in, parameter #2
    r*:       SCITER_VALUE; -- out, retval
  END;

CONST
  SC_LOAD_DATA*       = 01H;

(**This notification indicates that external data (for example image) download process
 * completed.
 *
 * param lParam #LPSCN_DATA_LOADED
 *
 * This notifiaction is sent for each external resource used by document when
 * this resource has been completely downloaded. Sciter will send this
 * notification asynchronously.
 **)
  SC_DATA_LOADED*     = 02H;

(**This notification is sent when all external data (for example image) has been downloaded.
 *
 * This notification is sent when all external resources required by document
 * have been completely downloaded. Sciter will send this notification
 * asynchronously.
 **)
  SC_DOCUMENT_COMPLETE* = 03H;

(**This notification is sent on parsing the document and while processing
 * elements having non empty style.behavior attribute value.
 *
 * param lParam #LPSCN_ATTACH_BEHAVIOR
 *
 * Application has to provide implementation of #sciter::behavior interface.
 * Set #SCN_ATTACH_BEHAVIOR::impl to address of this implementation.
 **)
  SC_ATTACH_BEHAVIOR* = 04H;

(**This notification is sent on
 * 1) stdin, stdout and stderr operations and
 * 2) view.hostCallback(p1,p2) calls from script
 *
 * param lParam #LPSCN_CALLBACK_HOST
 *
 **)
  SC_CALLBACK_HOST* = 05H;

TYPE
  PSciterHostCallback* = PROCEDURE ["StdCall"] (pns: Windows.PVOID; callbackParam: Windows.PVOID): Windows.UINT;

CONST
  SIH_REPLACE_CONTENT*     = 0;
  SIH_INSERT_AT_START*     = 1;
  SIH_APPEND_AFTER_LAST*   = 2;

  SOH_REPLACE*             = 3;
  SOH_INSERT_BEFORE*       = 4;
  SOH_INSERT_AFTER*        = 5;

TYPE
  HVM* = POINTER ["StdCall"] TO tiscript_VM;
  tiscript_VM* = RECORD ["StdCall"]
  END;

  SciterNativeMethod_t* = PROCEDURE ["StdCall"] (hvm: HVM; selfp: PSCITER_VALUE; argv: PSCITER_VALUE; argc: SYSTEM.INT32; retval: PSCITER_VALUE); 

  SciterNativeProperty_t* = PROCEDURE ["StdCall"] (hvm: HVM; selfp: PSCITER_VALUE; sets: SYSTEM.INT32; val: PSCITER_VALUE);

  SciterNativeDtor_t* = PROCEDURE ["StdCall"] (hvm: HVM; p_data_slot_value: SYSTEM.ADDRESS);

  PSciterNativeMethodDef* = POINTER ["StdCall"] TO SciterNativeMethodDef;
  SciterNativeMethodDef*  = RECORD  ["StdCall"]
      name*: Windows.PCSTR;
      method*: SciterNativeMethod_t;
  END;

  PSciterNativePropertyDef* = POINTER ["StdCall"] TO SciterNativePropertyDef;
  SciterNativePropertyDef*  = RECORD  
      name*: Windows.PCSTR;
      propertys*: SciterNativeProperty_t;
  END;

  PSciterNativeConstantDef* = POINTER ["StdCall"] TO SciterNativeConstantDef;
  SciterNativeConstantDef*  = RECORD  
      name*: Windows.PCSTR;
      value*: SCITER_VALUE;
--      SciterNativeConstantDef( LPCSTR n,SCITER_VALUE v = SCITER_VALUE() ):name(n),value(v) {}
  END;

  PSciterNativeClassDef* = POINTER ["StdCall"] TO SciterNativeClassDef;
  SciterNativeClassDef*  = RECORD  
      name*: Windows.PCSTR;
      methods*: PSciterNativeMethodDef;
      properties*: PSciterNativePropertyDef;
      dtor*: SciterNativeDtor_t;
      constants*: PSciterNativeConstantDef;
  END;

  PROCEDURE ["StdCall"] / SciterClassNameA* (): Windows.PCSTR;
  PROCEDURE ["StdCall"] / SciterClassNameW* (): Windows.PCWSTR;
  PROCEDURE ["StdCall"] / SciterGetVM* (handle: Windows.HWND): HVM;
  PROCEDURE ["StdCall"] / SciterNativeDefineClass* ( hvm: HVM; pClassDef: PSciterNativeClassDef): SYSTEM.INT32;
  PROCEDURE ["StdCall"] / SciterSetExceptionValue* ( hvm: HVM; errorMsg: Windows.PCWSTR): SYSTEM.INT32;
  PROCEDURE ["StdCall"] / SciterVmEval* ( hvm: HVM; script: Windows.PCWSTR; scriptLength: Windows.UINT; pretval: PSCITER_VALUE): SYSTEM.INT32;
  PROCEDURE ["StdCall"] / SciterCreateObject* (hvm: HVM; className: Windows.PCWSTR; pretval: PSCITER_VALUE): SYSTEM.INT32;
  PROCEDURE ["StdCall"] / BasicHostCallback* (pns: SYSTEM.ADDRESS; callbackParam: SYSTEM.ADDRESS): Windows.UINT; 

(* SCITE Section *)

  PROCEDURE ["StdCall"] / SciterProcND*(hwnd: Windows.HWND; msg: Windows.UINT; wParam: Windows.WPARAM; lParam: Windows.LPARAM; VAR pbHandled: SYSTEM.INT32): Windows.LRESULT;
  PROCEDURE ["StdCall"] / SciterCall*(hWnd: Windows.HWND; functionName: Windows.PCSTR; argc: Windows.UINT; argv: PSCITER_VALUE; retval: PSCITER_VALUE): SYSTEM.INT32; 
  PROCEDURE ["StdCall"] / SciterSetCallback*(hWndSciter: Windows.HWND; cb: PSciterHostCallback; cbParam: SYSTEM.ADDRESS); 
  PROCEDURE ["StdCall"] / SciterLoadHtml*(hWndSciter: Windows.HWND; html: Windows.PBYTE; htmlSize: Windows.UINT; baseUrl: Windows.PCWSTR): SYSTEM.INT32;
  PROCEDURE ["StdCall"] / SciterLoadFile*(hWndSciter:Windows.HWND; filename: Windows.PCWSTR): SYSTEM.INT32;
  PROCEDURE ["StdCall"] / SciterEval*(hwnd: Windows.HWND; script: Windows.PCWSTR; scriptLength: Windows.UINT; pretval: PSCITER_VALUE): SYSTEM.INT32;
  
  PROCEDURE ["StdCall"] / SciterTranslateMessage*(VAR Msg: Windows.MSG): Windows.BOOL;

  PROCEDURE ["StdCall"] / ValueInit*(pval: PSCITER_VALUE): Windows.UINT; 
  PROCEDURE ["StdCall"] / ValueStringData*(pval: PSCITER_VALUE; VAR pChars: Windows.PCWSTR; VAR pNumChars: Windows.UINT): Windows.UINT; 
  PROCEDURE ["StdCall"] / ValueStringDataSet*(pval: PSCITER_VALUE; Chars: Windows.PCWSTR; NumChars: Windows.UINT; units: Windows.UINT): Windows.UINT; 


(* HTMLayout Section *)

  PROCEDURE ["StdCall"] /  HTMLayoutProcND*(hwnd:Windows.HWND; msg: Windows.UINT; wParam: Windows.WPARAM; lParam: Windows.LPARAM;  VAR pbHandled: SYSTEM.INT32): SYSTEM.INT32;
  PROCEDURE ["StdCall"] /  HTMLayoutLoadHtml*(hwnd:Windows.HWND; html:Windows.PBYTE; htmlSize: Windows.UINT): SYSTEM.INT32;

  PROCEDURE ["StdCall"] /  HTMLayoutSetCallback*(hWndHTMLayout:Windows.HWND; cb: THTMLAYOUT_NOTIFY; cbParam: SYSTEM.INT32);

  PROCEDURE ["StdCall"] /  HTMLayoutGetRootElement*(hwnd: Windows.HWND; VAR phe: HELEMENT): Windows.LRESULT;
  PROCEDURE ["StdCall"] /  HTMLayoutGetChildrenCount*(he: HELEMENT; VAR count: Windows.UINT): Windows.LRESULT;
  PROCEDURE ["StdCall"] /  HTMLayoutGetNthChild*(he: HELEMENT; n: Windows.UINT; VAR phe: HELEMENT): Windows.LRESULT;

  PROCEDURE ["StdCall"] /  HTMLayoutGetElementText*(he: HELEMENT; characters: Windows.PWSTR; VAR length: Windows.UINT): Windows.LRESULT;
  PROCEDURE ["StdCall"] /  HTMLayoutGetElementHtml*(he: HELEMENT; VAR utf8bytes: Windows.PBYTE; outer: SYSTEM.INT32): Windows.LRESULT;
  PROCEDURE ["StdCall"] /  HTMLayoutSetElementHtml*(he: HELEMENT; html: Windows.PCSTR; htmlLength: Windows.DWORD; where: Windows.UINT): Windows.LRESULT;
  PROCEDURE ["StdCall"] /  HTMLayoutGetElementInnerText16*(he: HELEMENT; VAR utf16words: Windows.PWSTR): Windows.LRESULT;
  PROCEDURE ["StdCall"] /  HTMLayoutSetElementInnerText16*(he: HELEMENT; utf16words: Windows.PCWSTR; length: Windows.UINT): Windows.LRESULT;

  PROCEDURE ["StdCall"] /  HTMLayoutGetElementType*(he: HELEMENT; VAR p_type: Windows.PCSTR): Windows.LRESULT;

  PROCEDURE ["StdCall"] /  HTMLayoutDeleteElement*(he: HELEMENT): Windows.LRESULT;
  PROCEDURE ["StdCall"] /  HTMLayoutCreateElement*(tagname: Windows.PCSTR; textOrNull: Windows.PCWSTR; VAR phe: HELEMENT): Windows.LRESULT;
  PROCEDURE ["StdCall"] /  HTMLayoutInsertElement*(he: HELEMENT; hparent: HELEMENT; index: Windows.UINT): Windows.LRESULT;
  PROCEDURE ["StdCall"] /  HTMLayoutDetachElement*(he: HELEMENT): Windows.LRESULT;
  PROCEDURE ["StdCall"] /  HTMLayoutCloneElement*(he: HELEMENT; VAR phe: HELEMENT): Windows.LRESULT;

  PROCEDURE ["StdCall"] /  HTMLayoutMoveElement*(he: HELEMENT; xView: SYSTEM.INT32; yView: SYSTEM.INT32): Windows.LRESULT;





END SciterDLL.